package ctbrec.ui.sites.stripchat;

import java.io.IOException;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ctbrec.sites.stripchat.Stripchat;
import ctbrec.sites.stripchat.StripchatHttpClient;
import ctbrec.ui.controls.Dialogs;
import ctbrec.ui.sites.AbstractSiteUi;
import ctbrec.ui.sites.ConfigUI;
import ctbrec.ui.tabs.TabProvider;
import javafx.application.Platform;

public class StripchatSiteUi extends AbstractSiteUi {

    private static final Logger LOG = LoggerFactory.getLogger(StripchatSiteUi.class);

    private StripchatTabProvider tabProvider;
    private StripchatConfigUI configUi;
    private Stripchat site;

    public StripchatSiteUi(Stripchat stripchat) {
        this.site = stripchat;
        tabProvider = new StripchatTabProvider(stripchat);
        configUi = new StripchatConfigUI(stripchat);
    }

    @Override
    public TabProvider getTabProvider() {
        return tabProvider;
    }

    @Override
    public ConfigUI getConfigUI() {
        return configUi;
    }

    @Override
    public synchronized boolean login() throws IOException {
        boolean automaticLogin = site.login();
        if (automaticLogin) {
            return true;
        } else {

            BlockingQueue<Boolean> queue = new LinkedBlockingQueue<>();

            Runnable showDialog = () -> {
                // login with external browser
                try {
                    new StripchatElectronLoginDialog(site.getHttpClient().getCookieJar());
                } catch (Exception e1) {
                    LOG.error("Error logging in with external browser", e1);
                    Dialogs.showError("Login error", "Couldn't login to " + site.getName(), e1);
                }

                try {
                    queue.put(true);
                } catch (InterruptedException e) {
                    LOG.error("Error while signaling termination", e);
                }
            };

            Platform.runLater(showDialog);
            try {
                queue.take();
            } catch (InterruptedException e) {
                LOG.error("Error while waiting for login dialog to close", e);
                throw new IOException(e);
            }

            StripchatHttpClient httpClient = (StripchatHttpClient) site.getHttpClient();
            boolean loggedIn = httpClient.checkLoginSuccess();
            return loggedIn;
        }
    }
}
